@@ -12,13 +12,8 @@ class AutoGain : public PluginBase {
 
   GstElement* autogain = nullptr;
 
-  sigc::signal<void, float> momentary;
-  sigc::signal<void, float> shortterm;
-  sigc::signal<void, float> integrated;
-  sigc::signal<void, float> relative;
-  sigc::signal<void, float> loudness;
-  sigc::signal<void, float> range;
-  sigc::signal<void, float> gain;
+  sigc::signal<void, float> momentary, shortterm, integrated, relative,
+      loudness, range, gain;
 
  private:
   void bind_to_gsettings();
