@@ -7,11 +7,12 @@ class PitchPreset : public PluginPresetBase {
  public:
   PitchPreset();
 
-  void write(boost::property_tree::ptree& root) override;
-  void read(boost::property_tree::ptree& root) override;
+  void write(PresetType preset_type,
+             boost::property_tree::ptree& root) override;
+  void read(PresetType preset_type, boost::property_tree::ptree& root) override;
 
  private:
-  Glib::RefPtr<Gio::Settings> input_settings;
+  Glib::RefPtr<Gio::Settings> input_settings, output_settings;
 
   void save(boost::property_tree::ptree& root,
             const std::string& section,
