@@ -15,6 +15,11 @@ struct myServerInfo {
   std::string server_version;
   std::string default_sink_name;
   std::string default_source_name;
+  std::string protocol;
+  std::string format;
+  std::string channel_map;
+  uint rate;
+  uint8_t channels;
 };
 
 struct mySinkInfo {
@@ -26,6 +31,7 @@ struct mySinkInfo {
   std::string monitor_source_name;
   uint rate;
   std::string format;
+  std::string active_port;
 };
 
 struct mySourceInfo {
@@ -34,6 +40,19 @@ struct mySourceInfo {
   std::string description;
   uint rate;
   std::string format;
+  std::string active_port;
+};
+
+struct myModuleInfo {
+  std::string name;
+  uint index;
+  std::string argument;
+};
+
+struct myClientInfo {
+  std::string name;
+  uint index;
+  std::string binary;
 };
 
 struct AppInfo {
@@ -93,10 +112,15 @@ class PulseManager {
                                 uint value);
   void set_source_output_mute(const std::string& name, uint idx, bool state);
   void get_sink_input_info(uint idx);
+  void update_server_info(const pa_server_info* info);
+  void get_modules_info();
+  void get_clients_info();
 
   sigc::signal<void, std::shared_ptr<mySourceInfo>> source_added;
+  sigc::signal<void, std::shared_ptr<mySourceInfo>> source_changed;
   sigc::signal<void, uint> source_removed;
   sigc::signal<void, std::shared_ptr<mySinkInfo>> sink_added;
+  sigc::signal<void, std::shared_ptr<mySinkInfo>> sink_changed;
   sigc::signal<void, uint> sink_removed;
   sigc::signal<void, std::string> new_default_sink;
   sigc::signal<void, std::string> new_default_source;
@@ -106,6 +130,9 @@ class PulseManager {
   sigc::signal<void, std::shared_ptr<AppInfo>> source_output_added;
   sigc::signal<void, std::shared_ptr<AppInfo>> source_output_changed;
   sigc::signal<void, uint> source_output_removed;
+  sigc::signal<void> server_changed;
+  sigc::signal<void, std::shared_ptr<myModuleInfo>> module_info;
+  sigc::signal<void, std::shared_ptr<myClientInfo>> client_info;
 
  private:
   std::string log_tag = "pulse_manager: ";
@@ -234,7 +261,7 @@ class PulseManager {
       if (info->resample_method) {
         ai->resampler = info->resample_method;
       } else {
-        ai->resampler = "null";
+        ai->resampler = "none";
       }
 
       ai->format = pa_sample_format_to_string(info->sample_spec.format);
