@@ -1,8 +1,8 @@
+#include "application.hpp"
 #include <glibmm.h>
 #include <glibmm/i18n.h>
 #include <gtkmm/dialog.h>
 #include <gtkmm/messagedialog.h>
-#include "application.hpp"
 #include "application_ui.hpp"
 #include "config.h"
 #include "pulse_manager.hpp"
@@ -51,18 +51,27 @@ int Application::on_command_line(
   } else if (options->contains("presets")) {
     std::string list;
 
-    for (auto name : presets_manager->get_names()) {
+    for (auto name : presets_manager->get_names(PresetType::output)) {
       list += name + ",";
     }
 
-    util::info(log_tag + _("Presets: ") + list);
+    util::info(log_tag + _("Output Presets: ") + list);
+
+    list = "";
+
+    for (auto name : presets_manager->get_names(PresetType::input)) {
+      list += name + ",";
+    }
+
+    util::info(log_tag + _("Input Presets: ") + list);
   } else if (options->contains("load-preset")) {
     Glib::ustring name;
 
     if (!options->lookup_value("load-preset", name)) {
       util::debug(log_tag + "failed to load preset: " + name);
     } else {
-      presets_manager->load(name);
+      presets_manager->load(PresetType::input, name);
+      presets_manager->load(PresetType::output, name);
     }
   } else if (options->contains("reset")) {
     settings->reset("");
@@ -99,13 +108,43 @@ void Application::on_startup() {
 
   pm->new_default_sink.connect([&](auto name) {
     util::debug("new default sink: " + name);
-    sie->set_output_sink_name(name);
-    soe->webrtc->set_probe_src_device(name + ".monitor");
+
+    if (name != "") {
+      sie->set_output_sink_name(name);
+      soe->webrtc->set_probe_src_device(name + ".monitor");
+
+      auto info = pm->get_sink_info(name);
+      auto port = info->active_port;
+      std::string dev_name;
+
+      if (port != "null") {
+        dev_name = name + ":" + port;
+      } else {
+        dev_name = name;
+      }
+
+      presets_manager->autoload(PresetType::output, dev_name);
+    }
   });
 
   pm->new_default_source.connect([&](auto name) {
     util::debug("new default source: " + name);
-    soe->set_source_monitor_name(name);
+
+    if (name != "") {
+      soe->set_source_monitor_name(name);
+
+      auto info = pm->get_source_info(name);
+      auto port = info->active_port;
+      std::string dev_name;
+
+      if (port != "null") {
+        dev_name = name + ":" + port;
+      } else {
+        dev_name = name;
+      }
+
+      presets_manager->autoload(PresetType::input, dev_name);
+    }
   });
 
   settings->signal_changed("blacklist-in").connect([=](auto key) {
@@ -134,7 +173,16 @@ void Application::on_activate() {
 
     add_window(*window);
 
-    window->signal_hide().connect([&, window]() { delete window; });
+    window->signal_hide().connect([&, window]() {
+      int width, height;
+
+      window->get_size(width, height);
+
+      settings->set_int("window-width", width);
+      settings->set_int("window-height", height);
+
+      delete window;
+    });
 
     window->show_all();
 
@@ -203,7 +251,14 @@ void Application::create_actions() {
     withdraw_notification("reset");
   });
 
+  add_action("quit", [&] {
+    auto window = get_active_window();
+
+    window->hide();
+  });
+
   set_accel_for_action("app.help", "F1");
+  set_accel_for_action("app.quit", "<Ctrl>Q");
 }
 
 void Application::check_version() {
