@@ -1,9 +1,9 @@
+#include "convolver_ui.hpp"
 #include <glibmm.h>
 #include <glibmm/i18n.h>
 #include <gst/fft/gstfftf32.h>
 #include <boost/math/interpolators/cubic_b_spline.hpp>
 #include <sndfile.hh>
-#include "convolver_ui.hpp"
 
 ConvolverUi::ConvolverUi(BaseObjectType* cobject,
                          const Glib::RefPtr<Gtk::Builder>& builder,
@@ -11,7 +11,8 @@ ConvolverUi::ConvolverUi(BaseObjectType* cobject,
     : Gtk::Grid(cobject),
       PluginUiBase(builder, settings_name),
       irs_dir(Glib::get_user_config_dir() + "/PulseEffects/irs"),
-      global_settings(Gio::Settings::create("com.github.wwmm.pulseeffects")) {
+      spectrum_settings(
+          Gio::Settings::create("com.github.wwmm.pulseeffects.spectrum")) {
   name = "convolver";
 
   // loading glade widgets
@@ -63,12 +64,6 @@ ConvolverUi::ConvolverUi(BaseObjectType* cobject,
   irs_listbox->set_sort_func(
       sigc::mem_fun(*this, &ConvolverUi::on_listbox_sort));
 
-  irs_listbox->signal_row_activated().connect([&](auto row) {
-    auto irs_file = irs_dir / boost::filesystem::path{row->get_name() + ".irs"};
-
-    settings->set_string("kernel-path", irs_file.string());
-  });
-
   import_irs->signal_clicked().connect(
       sigc::mem_fun(*this, &ConvolverUi::on_import_irs_clicked));
 
@@ -102,7 +97,6 @@ ConvolverUi::ConvolverUi(BaseObjectType* cobject,
       util::warning(log_tag +
                     "failed to create irs directory: " + irs_dir.string());
     }
-
   } else {
     util::debug(log_tag + "irs directory already exists: " + irs_dir.string());
   }
@@ -122,25 +116,20 @@ ConvolverUi::ConvolverUi(BaseObjectType* cobject,
      is loaded
   */
 
-  settings->signal_changed("kernel-path").connect([=](auto key) {
-    auto f = [=]() {
-      std::lock_guard<std::mutex> lock(lock_guard_irs_info);
-      get_irs_info();
-    };
+  connections.push_back(
+      settings->signal_changed("kernel-path").connect([=](auto key) {
+        auto f = [=]() {
+          std::lock_guard<std::mutex> lock(lock_guard_irs_info);
+          get_irs_info();
+        };
 
-    auto future = std::async(std::launch::async, f);
+        auto future = std::async(std::launch::async, f);
 
-    futures.push_back(std::move(future));
-  });
+        futures.push_back(std::move(future));
+      }));
 }
 
 ConvolverUi::~ConvolverUi() {
-  settings->set_boolean("post-messages", false);
-
-  for (auto c : connections) {
-    c.disconnect();
-  }
-
   futures.clear();
 
   util::debug(name + " ui destroyed");
@@ -230,11 +219,12 @@ void ConvolverUi::populate_irs_listbox() {
         "/com/github/wwmm/pulseeffects/ui/irs_row.glade");
 
     Gtk::ListBoxRow* row;
-    Gtk::Button* remove_btn;
+    Gtk::Button *remove_btn, *apply_btn;
     Gtk::Label* label;
 
     b->get_widget("irs_row", row);
     b->get_widget("remove", remove_btn);
+    b->get_widget("apply", apply_btn);
     b->get_widget("name", label);
 
     row->set_name(name);
@@ -245,6 +235,13 @@ void ConvolverUi::populate_irs_listbox() {
       populate_irs_listbox();
     }));
 
+    connections.push_back(apply_btn->signal_clicked().connect([=]() {
+      auto irs_file =
+          irs_dir / boost::filesystem::path{row->get_name() + ".irs"};
+
+      settings->set_string("kernel-path", irs_file.string());
+    }));
+
     irs_listbox->add(*row);
     irs_listbox->show_all();
   }
@@ -394,6 +391,8 @@ void ConvolverUi::get_irs_info() {
     right_mag[n] = (right_mag[n] - min_right) / (max_right - min_right);
   }
 
+  get_irs_spectrum(rate);
+
   // updating interface with ir file info
 
   Glib::signal_idle().connect_once([=]() {
@@ -410,13 +409,11 @@ void ConvolverUi::get_irs_info() {
     auto fpath = boost::filesystem::path{path};
 
     label_file_name->set_text(fpath.stem().string());
+
+    left_plot->queue_draw();
+    right_plot->queue_draw();
   });
 
-  get_irs_spectrum(rate);
-
-  left_plot->queue_draw();
-  right_plot->queue_draw();
-
   delete[] kernel;
 }
 
@@ -557,9 +554,9 @@ void ConvolverUi::draw_channel(Gtk::DrawingArea* da,
       ctx->line_to(x[n + 1], height - bar_height);
     }
 
-    if (global_settings->get_boolean("use-custom-color")) {
+    if (spectrum_settings->get_boolean("use-custom-color")) {
       Glib::Variant<std::vector<double>> v;
-      global_settings->get_value("spectrum-color", v);
+      spectrum_settings->get_value("color", v);
       auto rgba = v.get();
 
       ctx->set_source_rgba(rgba[0], rgba[1], rgba[2], rgba[3]);
