@@ -1,10 +1,9 @@
+#include "pipeline_base.hpp"
 #include <glib-object.h>
 #include <gobject/gvaluecollector.h>
-#include <algorithm>
+#include <sys/resource.h>
 #include <boost/math/interpolators/cubic_b_spline.hpp>
-#include <cmath>
-#include <mutex>
-#include "pipeline_base.hpp"
+#include "config.h"
 #include "util.hpp"
 
 namespace {
@@ -26,6 +25,52 @@ void on_message_error(const GstBus* gst_bus,
   g_free(debug);
 }
 
+static void on_stream_status(GstBus* bus,
+                             GstMessage* message,
+                             PipelineBase* pb) {
+  GstStreamStatusType type;
+  GstElement* owner;
+  gchar* path;
+  std::string path_str, source_name;
+  std::size_t idx;
+  int priority, niceness, priority_type;
+
+  gst_message_parse_stream_status(message, &type, &owner);
+
+  switch (type) {
+    case GST_STREAM_STATUS_TYPE_CREATE:
+      break;
+    case GST_STREAM_STATUS_TYPE_ENTER:
+      path = gst_object_get_path_string(GST_OBJECT(owner));
+
+      path_str = path;
+
+      idx = path_str.find_last_of("/");
+
+      source_name = path_str.substr(idx + 1);
+
+      g_free(path);
+
+      priority_type = g_settings_get_enum(pb->settings, "priority-type");
+
+      if (priority_type == 0) {  // Niceness (high priority in rtkit terms)
+        niceness = g_settings_get_int(pb->settings, "niceness");
+
+        pb->rtkit->set_nice(source_name, niceness);
+      } else if (priority_type == 1) {  // Real Time
+        priority = g_settings_get_int(pb->settings, "realtime-priority");
+
+        pb->rtkit->set_priority(source_name, priority);
+      }
+
+      break;
+    case GST_STREAM_STATUS_TYPE_LEAVE:
+      break;
+    default:
+      break;
+  }
+}
+
 void on_message_state_changed(const GstBus* gst_bus,
                               GstMessage* message,
                               PipelineBase* pb) {
@@ -40,6 +85,7 @@ void on_message_state_changed(const GstBus* gst_bus,
 
     if (new_state == GST_STATE_PLAYING) {
       pb->playing = true;
+
       pb->get_latency();
     } else {
       pb->playing = false;
@@ -71,6 +117,8 @@ void on_message_latency(const GstBus* gst_bus,
     util::debug(pb->log_tag +
                 "pulsesink buffer [us]: " + std::to_string(buffer));
   }
+
+  pb->get_latency();
 }
 
 void on_message_element(const GstBus* gst_bus,
@@ -108,7 +156,11 @@ void on_message_element(const GstBus* gst_bus,
 
     if (max_mag > min_mag) {
       for (uint n = 0; n < pb->spectrum_mag.size(); n++) {
-        pb->spectrum_mag[n] = (min_mag - pb->spectrum_mag[n]) / min_mag;
+        if (min_mag < pb->spectrum_mag[n]) {
+          pb->spectrum_mag[n] = (min_mag - pb->spectrum_mag[n]) / min_mag;
+        } else {
+          pb->spectrum_mag[n] = 0.0f;
+        }
       }
 
       Glib::signal_idle().connect_once(
@@ -120,7 +172,7 @@ void on_message_element(const GstBus* gst_bus,
 void on_spectrum_n_points_changed(GSettings* settings,
                                   gchar* key,
                                   PipelineBase* pb) {
-  long unsigned int npoints = g_settings_get_int(settings, "spectrum-n-points");
+  long unsigned int npoints = g_settings_get_int(settings, "n-points");
 
   if (npoints != pb->spectrum_mag.size()) {
     pb->resizing_spectrum = true;
@@ -183,21 +235,43 @@ void on_latency_changed(GObject* gobject, GParamSpec* pspec, PipelineBase* pb) {
   }
 }
 
+GstPadProbeReturn on_sink_event(GstPad* pad,
+                                GstPadProbeInfo* info,
+                                gpointer user_data) {
+  GstEvent* event = GST_PAD_PROBE_INFO_EVENT(info);
+
+  if (event->type == GST_EVENT_LATENCY) {
+    auto pb = static_cast<PipelineBase*>(user_data);
+
+    pb->get_latency();
+  }
+
+  return GST_PAD_PROBE_OK;
+}
+
 }  // namespace
 
-PipelineBase::PipelineBase(const std::string& tag, const uint& sampling_rate)
-    : log_tag(tag), settings(g_settings_new("com.github.wwmm.pulseeffects")) {
+PipelineBase::PipelineBase(const std::string& tag, PulseManager* pulse_manager)
+    : log_tag(tag),
+      pm(pulse_manager),
+      settings(g_settings_new("com.github.wwmm.pulseeffects")),
+      spectrum_settings(
+          g_settings_new("com.github.wwmm.pulseeffects.spectrum")),
+      rtkit(std::make_unique<RealtimeKit>(tag)) {
   gst_init(nullptr, nullptr);
 
   pipeline = gst_pipeline_new("pipeline");
 
   bus = gst_element_get_bus(pipeline);
 
+  gst_bus_enable_sync_message_emission(bus);
   gst_bus_add_signal_watch(bus);
 
   // bus callbacks
 
   g_signal_connect(bus, "message::error", G_CALLBACK(on_message_error), this);
+  g_signal_connect(bus, "sync-message::stream-status",
+                   GCallback(on_stream_status), this);
   g_signal_connect(bus, "message::state-changed",
                    G_CALLBACK(on_message_state_changed), this);
   g_signal_connect(bus, "message::latency", G_CALLBACK(on_message_latency),
@@ -207,16 +281,16 @@ PipelineBase::PipelineBase(const std::string& tag, const uint& sampling_rate)
 
   // creating elements common to all pipelines
 
-  source = gst_element_factory_make("pulsesrc", "source");
-  capsfilter = gst_element_factory_make("capsfilter", nullptr);
-  adapter = gst_element_factory_make("peadapter", nullptr);
-  sink = gst_element_factory_make("pulsesink", "sink");
-  spectrum = gst_element_factory_make("spectrum", "spectrum");
+  gst_registry_scan_path(gst_registry_get(), PLUGINS_INSTALL_DIR);
 
-  auto queue_src = gst_element_factory_make("queue", nullptr);
-  auto src_type = gst_element_factory_make("typefind", nullptr);
-  auto audioconvert_in = gst_element_factory_make("audioconvert", nullptr);
-  auto audioconvert_out = gst_element_factory_make("audioconvert", nullptr);
+  source = get_required_plugin("pulsesrc", "source");
+  queue_src = get_required_plugin("queue", nullptr);
+  capsfilter = get_required_plugin("capsfilter", nullptr);
+  sink = get_required_plugin("pulsesink", "sink");
+  spectrum = get_required_plugin("spectrum", "spectrum");
+  adapter = gst_element_factory_make("peadapter", nullptr);
+
+  auto src_type = get_required_plugin("typefind", nullptr);
 
   init_spectrum_bin();
   init_effects_bin();
@@ -224,12 +298,10 @@ PipelineBase::PipelineBase(const std::string& tag, const uint& sampling_rate)
   // building the pipeline
 
   gst_bin_add_many(GST_BIN(pipeline), source, queue_src, capsfilter, src_type,
-                   audioconvert_in, adapter, audioconvert_out, effects_bin,
-                   spectrum_bin, sink, nullptr);
+                   adapter, effects_bin, spectrum_bin, sink, nullptr);
 
-  gst_element_link_many(source, queue_src, capsfilter, src_type,
-                        audioconvert_in, adapter, audioconvert_out, effects_bin,
-                        spectrum_bin, sink, nullptr);
+  gst_element_link_many(source, queue_src, capsfilter, src_type, adapter,
+                        effects_bin, spectrum_bin, sink, nullptr);
 
   // initializing properties
 
@@ -244,6 +316,7 @@ PipelineBase::PipelineBase(const std::string& tag, const uint& sampling_rate)
   g_object_set(sink, "provide-clock", true, nullptr);
 
   g_object_set(queue_src, "silent", true, nullptr);
+  g_object_set(queue_src, "flush-on-eos", true, nullptr);
   g_object_set(queue_src, "max-size-buffers", 0, nullptr);
   g_object_set(queue_src, "max-size-bytes", 0, nullptr);
   g_object_set(queue_src, "max-size-time", 0, nullptr);
@@ -251,14 +324,19 @@ PipelineBase::PipelineBase(const std::string& tag, const uint& sampling_rate)
   g_object_set(spectrum, "bands", spectrum_nbands, nullptr);
   g_object_set(spectrum, "threshold", spectrum_threshold, nullptr);
 
-  set_caps(sampling_rate);
-
   g_signal_connect(src_type, "have-type", G_CALLBACK(on_src_type_changed),
                    this);
   g_signal_connect(source, "notify::buffer-time", G_CALLBACK(on_buffer_changed),
                    this);
   g_signal_connect(source, "notify::latency-time",
                    G_CALLBACK(on_latency_changed), this);
+
+  auto sinkpad = gst_element_get_static_pad(sink, "sink");
+
+  gst_pad_add_probe(sinkpad, GST_PAD_PROBE_TYPE_EVENT_UPSTREAM, on_sink_event,
+                    this, nullptr);
+
+  g_object_unref(sinkpad);
 }
 
 PipelineBase::~PipelineBase() {
@@ -276,9 +354,13 @@ PipelineBase::~PipelineBase() {
   gst_object_unref(bus);
   gst_object_unref(pipeline);
   g_object_unref(settings);
+  g_object_unref(spectrum_settings);
+  g_object_unref(child_settings);
 }
 
 void PipelineBase::set_caps(const uint& sampling_rate) {
+  current_rate = sampling_rate;
+
   auto caps_str = "audio/x-raw,format=F32LE,channels=2,rate=" +
                   std::to_string(sampling_rate);
 
@@ -376,7 +458,7 @@ void PipelineBase::set_null_pipeline() {
   gst_element_get_state(pipeline, &state, &pending, state_check_timeout);
 
   /*on_message_state is not called when going to null. I don't know why.
-   *so we have to update the variable manually after setting to null.
+   * so we have to update the variable manually after setting to null.
    */
 
   if (state == GST_STATE_NULL) {
@@ -457,8 +539,33 @@ void PipelineBase::on_app_removed(uint idx) {
   update_pipeline_state();
 }
 
+void PipelineBase::on_sink_changed(std::shared_ptr<mySinkInfo> sink_info) {
+  if (sink_info->name == "PulseEffects_apps") {
+    if (sink_info->rate != current_rate) {
+      gst_element_set_state(pipeline, GST_STATE_READY);
+
+      set_caps(sink_info->rate);
+
+      update_pipeline_state();
+    }
+  }
+}
+
+void PipelineBase::on_source_changed(
+    std::shared_ptr<mySourceInfo> source_info) {
+  if (source_info->name == "PulseEffects_mic.monitor") {
+    if (source_info->rate != current_rate) {
+      gst_element_set_state(pipeline, GST_STATE_READY);
+
+      set_caps(source_info->rate);
+
+      update_pipeline_state();
+    }
+  }
+}
+
 void PipelineBase::init_spectrum(const uint& sampling_rate) {
-  g_signal_connect(settings, "changed::spectrum-n-points",
+  g_signal_connect(spectrum_settings, "changed::n-points",
                    G_CALLBACK(on_spectrum_n_points_changed), this);
 
   spectrum_freqs.clear();
@@ -477,7 +584,7 @@ void PipelineBase::init_spectrum(const uint& sampling_rate) {
 
   spectrum_mag_tmp.resize(spectrum_freqs.size());
 
-  auto npoints = g_settings_get_int(settings, "spectrum-n-points");
+  auto npoints = g_settings_get_int(spectrum_settings, "n-points");
 
   spectrum_x_axis = util::logspace(log10(min_spectrum_freq),
                                    log10(max_spectrum_freq), npoints);
@@ -488,6 +595,12 @@ void PipelineBase::init_spectrum(const uint& sampling_rate) {
   spline_df = spectrum_freqs[1] - spectrum_freqs[0];
 }
 
+void PipelineBase::update_spectrum_interval(const double& value) {
+  auto interval = guint64(1000000000. / value);
+
+  g_object_set(spectrum, "interval", interval, nullptr);
+}
+
 void PipelineBase::enable_spectrum() {
   auto srcpad = gst_element_get_static_pad(spectrum_identity_in, "src");
 
@@ -583,3 +696,14 @@ std::array<double, 2> PipelineBase::get_peak(GstMessage* message) {
 
   return peak;
 }
+
+GstElement* PipelineBase::get_required_plugin(const gchar* factoryname,
+                                              const gchar* name) {
+  GstElement* plugin = gst_element_factory_make(factoryname, name);
+
+  if (!plugin)
+    throw std::runtime_error(
+        log_tag + std::string("Failed to get required plugin: ") + factoryname);
+
+  return plugin;
+}
