@@ -1,30 +1,29 @@
+#include "pulse_manager.hpp"
 #include <glibmm.h>
 #include <memory>
-#include "pulse_manager.hpp"
 #include "util.hpp"
 
 PulseManager::PulseManager()
     : main_loop(pa_threaded_mainloop_new()),
-      main_loop_api(pa_threaded_mainloop_get_api(main_loop)),
-      context(pa_context_new(main_loop_api, "PulseEffects")) {
-  pa_context_set_state_callback(context, &PulseManager::context_state_cb, this);
-
-  pa_context_connect(context, nullptr, PA_CONTEXT_NOFAIL, nullptr);
-
+      main_loop_api(pa_threaded_mainloop_get_api(main_loop)) {
+  pa_threaded_mainloop_lock(main_loop);
   pa_threaded_mainloop_start(main_loop);
 
-  pa_threaded_mainloop_lock(main_loop);
-
-  while (!context_ready) {
-    pa_threaded_mainloop_wait(main_loop);
-  }
+  context = pa_context_new(main_loop_api, "PulseEffects");
 
+  pa_context_set_state_callback(context, &PulseManager::context_state_cb, this);
+  pa_context_connect(context, nullptr, PA_CONTEXT_NOFAIL, nullptr);
+  pa_threaded_mainloop_wait(main_loop);
   pa_threaded_mainloop_unlock(main_loop);
 
-  get_server_info();
-  load_apps_sink();
-  load_mic_sink();
-  subscribe_to_events();
+  if (context_ready) {
+    get_server_info();
+    load_apps_sink();
+    load_mic_sink();
+    subscribe_to_events();
+  } else {
+    util::error(log_tag + "context initialization failed");
+  }
 }
 
 PulseManager::~PulseManager() {
@@ -37,20 +36,16 @@ PulseManager::~PulseManager() {
   util::debug(log_tag + "disconnecting Pulseaudio context...");
   pa_context_disconnect(context);
 
-  while (context_ready) {
-    pa_threaded_mainloop_wait(main_loop);
-  }
-
-  pa_threaded_mainloop_unlock(main_loop);
-
   util::debug(log_tag + "Pulseaudio context was disconnected");
 
-  util::debug(log_tag + "stopping pulseaudio threaded main loop");
-  pa_threaded_mainloop_stop(main_loop);
-
   util::debug(log_tag + "unreferencing Pulseaudio context");
   pa_context_unref(context);
 
+  pa_threaded_mainloop_unlock(main_loop);
+
+  util::debug(log_tag + "stopping pulseaudio threaded main loop");
+  pa_threaded_mainloop_stop(main_loop);
+
   util::debug(log_tag + "freeing Pulseaudio threaded main loop");
   pa_threaded_mainloop_free(main_loop);
 }
@@ -71,14 +66,19 @@ void PulseManager::context_state_cb(pa_context* ctx, void* data) {
   } else if (state == PA_CONTEXT_READY) {
     util::debug(pm->log_tag + "context is ready");
     util::debug(pm->log_tag + "connected to: " + pa_context_get_server(ctx));
-    util::debug(pm->log_tag + "protocol version: " +
-                std::to_string(pa_context_get_protocol_version(ctx)));
+
+    auto protocol = std::to_string(pa_context_get_protocol_version(ctx));
+
+    pm->server_info.protocol = protocol;
+
+    util::debug(pm->log_tag + "protocol version: " + protocol);
 
     pm->context_ready = true;
     pa_threaded_mainloop_signal(pm->main_loop, false);
   } else if (state == PA_CONTEXT_FAILED) {
     util::debug(pm->log_tag + "failed to connect context");
 
+    pm->context_ready = false;
     pa_threaded_mainloop_signal(pm->main_loop, false);
   } else if (state == PA_CONTEXT_TERMINATED) {
     util::debug(pm->log_tag + "context was terminated");
@@ -181,6 +181,32 @@ void PulseManager::subscribe_to_events() {
                 },
                 pm);
           } else if (e == PA_SUBSCRIPTION_EVENT_CHANGE) {
+            pa_context_get_source_info_by_index(
+                c, idx,
+                [](auto cx, auto info, auto eol, auto d) {
+                  if (info != nullptr) {
+                    auto pm = static_cast<PulseManager*>(d);
+
+                    auto si = std::make_shared<mySourceInfo>();
+
+                    si->name = info->name;
+                    si->index = info->index;
+                    si->description = info->description;
+                    si->rate = info->sample_spec.rate;
+                    si->format =
+                        pa_sample_format_to_string(info->sample_spec.format);
+
+                    if (si->name == "PulseEffects_mic.monitor") {
+                      pm->mic_sink_info->rate = si->rate;
+                      pm->mic_sink_info->format = si->format;
+                    }
+
+                    Glib::signal_idle().connect_once([pm, si = move(si)] {
+                      pm->source_changed.emit(move(si));
+                    });
+                  }
+                },
+                pm);
           } else if (e == PA_SUBSCRIPTION_EVENT_REMOVE) {
             Glib::signal_idle().connect_once(
                 [pm, idx]() { pm->source_removed.emit(idx); });
@@ -216,6 +242,31 @@ void PulseManager::subscribe_to_events() {
                 },
                 pm);
           } else if (e == PA_SUBSCRIPTION_EVENT_CHANGE) {
+            pa_context_get_sink_info_by_index(
+                c, idx,
+                [](auto cx, auto info, auto eol, auto d) {
+                  if (info != nullptr) {
+                    auto pm = static_cast<PulseManager*>(d);
+                    auto si = std::make_shared<mySinkInfo>();
+
+                    si->name = info->name;
+                    si->index = info->index;
+                    si->description = info->description;
+                    si->rate = info->sample_spec.rate;
+                    si->format =
+                        pa_sample_format_to_string(info->sample_spec.format);
+
+                    if (si->name == "PulseEffects_apps") {
+                      pm->apps_sink_info->rate = si->rate;
+                      pm->apps_sink_info->format = si->format;
+                    }
+
+                    Glib::signal_idle().connect_once([pm, si = move(si)] {
+                      pm->sink_changed.emit(move(si));
+                    });
+                  }
+                },
+                pm);
           } else if (e == PA_SUBSCRIPTION_EVENT_REMOVE) {
             Glib::signal_idle().connect_once(
                 [pm, idx]() { pm->sink_removed.emit(idx); });
@@ -230,14 +281,10 @@ void PulseManager::subscribe_to_events() {
                   if (info != nullptr) {
                     auto pm = static_cast<PulseManager*>(d);
 
-                    pm->server_info.server_name = info->server_name;
-                    pm->server_info.server_version = info->server_version;
+                    pm->update_server_info(info);
 
-                    auto sink = info->default_sink_name;
-                    auto source = info->default_source_name;
-
-                    pm->server_info.default_sink_name = sink;
-                    pm->server_info.default_source_name = source;
+                    std::string sink = info->default_sink_name;
+                    std::string source = info->default_source_name;
 
                     if (sink != std::string("PulseEffects_apps")) {
                       Glib::signal_idle().connect_once(
@@ -249,6 +296,9 @@ void PulseManager::subscribe_to_events() {
                         pm->new_default_source.emit(source);
                       });
                     }
+
+                    Glib::signal_idle().connect_once(
+                        [pm]() { pm->server_changed.emit(); });
                   }
                 },
                 pm);
@@ -274,6 +324,17 @@ void PulseManager::subscribe_to_events() {
       this);
 }
 
+void PulseManager::update_server_info(const pa_server_info* info) {
+  server_info.server_name = info->server_name;
+  server_info.server_version = info->server_version;
+  server_info.default_sink_name = info->default_sink_name;
+  server_info.default_source_name = info->default_source_name;
+  server_info.format = pa_sample_format_to_string(info->sample_spec.format);
+  server_info.rate = info->sample_spec.rate;
+  server_info.channels = info->sample_spec.channels;
+  server_info.channel_map = pa_channel_map_to_pretty_name(&info->channel_map);
+}
+
 void PulseManager::get_server_info() {
   pa_threaded_mainloop_lock(main_loop);
 
@@ -283,10 +344,7 @@ void PulseManager::get_server_info() {
         auto pm = static_cast<PulseManager*>(d);
 
         if (info != nullptr) {
-          pm->server_info.server_name = info->server_name;
-          pm->server_info.server_version = info->server_version;
-          pm->server_info.default_sink_name = info->default_sink_name;
-          pm->server_info.default_source_name = info->default_source_name;
+          pm->update_server_info(info);
 
           util::debug(pm->log_tag +
                       "Pulseaudio version: " + info->server_version);
@@ -345,6 +403,12 @@ std::shared_ptr<mySinkInfo> PulseManager::get_sink_info(std::string name) {
           d->si->monitor_source_name = info->monitor_source_name;
           d->si->rate = info->sample_spec.rate;
           d->si->format = pa_sample_format_to_string(info->sample_spec.format);
+
+          if (info->active_port != nullptr) {
+            d->si->active_port = info->active_port->name;
+          } else {
+            d->si->active_port = "null";
+          }
         }
       },
       &data);
@@ -397,6 +461,12 @@ std::shared_ptr<mySourceInfo> PulseManager::get_source_info(std::string name) {
           d->si->description = info->description;
           d->si->rate = info->sample_spec.rate;
           d->si->format = pa_sample_format_to_string(info->sample_spec.format);
+
+          if (info->active_port != nullptr) {
+            d->si->active_port = info->active_port->name;
+          } else {
+            d->si->active_port = "null";
+          }
         }
       },
       &data);
@@ -460,10 +530,31 @@ std::shared_ptr<mySinkInfo> PulseManager::load_sink(std::string name,
   auto si = get_sink_info(name);
 
   if (si == nullptr) {  // sink is not loaded
-    std::string argument = "sink_name=" + name + " " +
-                           "sink_properties=" + description +
-                           "device.class=\"sound\"" + " " + "channels=2" + " " +
-                           "rate=" + std::to_string(rate);
+    std::string argument;
+
+    int version = std::stoi(server_info.server_version);
+
+    if (server_info.server_version.find("-") != std::string::npos) {
+      /* The user is probably running a Pulseaudio compiled from git.
+         norewinds will be added to Pulseaudio 13. People running its
+         development branch 12.0-**** can use the option norewind.
+      */
+
+      if (version == 12) {
+        version = 13;
+      }
+    }
+
+    if (version >= 13) {
+      argument = "sink_name=" + name + " " + "sink_properties=" + description +
+                 "device.class=\"sound\"" + " " + "norewinds=1";
+
+      util::debug(log_tag + "Null sinks rewinds will be disabled");
+    } else {
+      argument = "sink_name=" + name + " " + "sink_properties=" + description +
+                 "device.class=\"sound\"" + " " + "channels=2" + " " +
+                 "rate=" + std::to_string(rate);
+    }
 
     pa_threaded_mainloop_lock(main_loop);
 
@@ -1084,6 +1175,98 @@ void PulseManager::get_sink_input_info(uint idx) {
   pa_threaded_mainloop_unlock(main_loop);
 }
 
+void PulseManager::get_modules_info() {
+  pa_threaded_mainloop_lock(main_loop);
+
+  auto o = pa_context_get_module_info_list(
+      context,
+      [](auto c, auto info, auto eol, auto d) {
+        auto pm = static_cast<PulseManager*>(d);
+
+        if (eol < 0) {
+          pa_threaded_mainloop_signal(pm->main_loop, false);
+        } else if (eol > 0) {
+          pa_threaded_mainloop_signal(pm->main_loop, false);
+        } else if (info != nullptr) {
+          auto mi = std::make_shared<myModuleInfo>();
+
+          if (info->name) {
+            mi->name = info->name;
+            mi->index = info->index;
+
+            if (info->argument) {
+              mi->argument = info->argument;
+            } else {
+              mi->argument = "";
+            }
+
+            Glib::signal_idle().connect_once(
+                [pm, mi = move(mi)] { pm->module_info.emit(move(mi)); });
+          }
+        }
+      },
+      this);
+
+  if (o != nullptr) {
+    while (pa_operation_get_state(o) == PA_OPERATION_RUNNING) {
+      pa_threaded_mainloop_wait(main_loop);
+    }
+
+    pa_operation_unref(o);
+  } else {
+    util::critical(log_tag + "failed to get modules info");
+  }
+
+  pa_threaded_mainloop_unlock(main_loop);
+}
+
+void PulseManager::get_clients_info() {
+  pa_threaded_mainloop_lock(main_loop);
+
+  auto o = pa_context_get_client_info_list(
+      context,
+      [](auto c, auto info, auto eol, auto d) {
+        auto pm = static_cast<PulseManager*>(d);
+
+        if (eol < 0) {
+          pa_threaded_mainloop_signal(pm->main_loop, false);
+        } else if (eol > 0) {
+          pa_threaded_mainloop_signal(pm->main_loop, false);
+        } else if (info != nullptr) {
+          auto mi = std::make_shared<myClientInfo>();
+
+          if (info->name) {
+            mi->name = info->name;
+            mi->index = info->index;
+
+            if (pa_proplist_contains(info->proplist,
+                                     "application.process.binary") == 1) {
+              mi->binary = pa_proplist_gets(info->proplist,
+                                            "application.process.binary");
+            } else {
+              mi->binary = "";
+            }
+
+            Glib::signal_idle().connect_once(
+                [pm, mi = move(mi)] { pm->client_info.emit(move(mi)); });
+          }
+        }
+      },
+      this);
+
+  if (o != nullptr) {
+    while (pa_operation_get_state(o) == PA_OPERATION_RUNNING) {
+      pa_threaded_mainloop_wait(main_loop);
+    }
+
+    pa_operation_unref(o);
+  } else {
+    util::critical(log_tag + "failed to get clients info");
+  }
+
+  pa_threaded_mainloop_unlock(main_loop);
+}
+
 void PulseManager::unload_module(uint idx) {
   struct Data {
     uint idx;
