@@ -1,5 +1,5 @@
 #include "sink_input_effects.hpp"
-#include "util.hpp"
+#include "pipeline_common.hpp"
 
 namespace {
 
@@ -8,10 +8,10 @@ void on_message_element(const GstBus* gst_bus,
                         SinkInputEffects* sie) {
   auto src_name = GST_OBJECT_NAME(message->src);
 
-  if (src_name == std::string("compressor_input_level")) {
-    sie->compressor_input_level.emit(sie->get_peak(message));
-  } else if (src_name == std::string("compressor_output_level")) {
-    sie->compressor_output_level.emit(sie->get_peak(message));
+  if (src_name == std::string("pitch_input_level")) {
+    sie->pitch_input_level.emit(sie->get_peak(message));
+  } else if (src_name == std::string("pitch_output_level")) {
+    sie->pitch_output_level.emit(sie->get_peak(message));
   } else if (src_name == std::string("equalizer_input_level")) {
     sie->equalizer_input_level.emit(sie->get_peak(message));
   } else if (src_name == std::string("equalizer_output_level")) {
@@ -56,196 +56,26 @@ void on_message_element(const GstBus* gst_bus,
     sie->autogain_input_level.emit(sie->get_peak(message));
   } else if (src_name == std::string("autogain_output_level")) {
     sie->autogain_output_level.emit(sie->get_peak(message));
-  }
-}
-
-void update_order(gpointer user_data) {
-  auto l = static_cast<SinkInputEffects*>(user_data);
-
-  if (!gst_element_is_locked_state(l->effects_bin)) {
-    if (!gst_element_set_locked_state(l->effects_bin, true)) {
-      util::debug(l->log_tag + " could not lock state changes");
-    }
-  }
-
-  // setting null state
-
-  for (auto& p : l->plugins) {
-    gst_element_set_state(p.second, GST_STATE_NULL);
-  }
-
-  // unlinking elements using old plugins order
-
-  gst_element_unlink(l->identity_in, l->plugins[l->plugins_order_old[0]]);
-
-  for (long unsigned int n = 1; n < l->plugins_order_old.size(); n++) {
-    gst_element_unlink(l->plugins[l->plugins_order_old[n - 1]],
-                       l->plugins[l->plugins_order_old[n]]);
-  }
-
-  gst_element_unlink(
-      l->plugins[l->plugins_order_old[l->plugins_order_old.size() - 1]],
-      l->identity_out);
-
-  // linking elements using the new plugins order
-
-  gst_element_link(l->identity_in, l->plugins[l->plugins_order[0]]);
-
-  for (long unsigned int n = 1; n < l->plugins_order.size(); n++) {
-    gst_element_link(l->plugins[l->plugins_order[n - 1]],
-                     l->plugins[l->plugins_order[n]]);
-  }
-
-  gst_element_link(l->plugins[l->plugins_order[l->plugins_order.size() - 1]],
-                   l->identity_out);
-
-  for (auto& p : l->plugins) {
-    gst_element_sync_state_with_parent(p.second);
-  }
-
-  gst_element_set_locked_state(l->effects_bin, false);
-
-  gst_element_sync_state_with_parent(l->effects_bin);
-
-  std::string list;
-
-  for (auto name : l->plugins_order) {
-    list += name + ",";
-  }
-
-  util::debug(l->log_tag + "new plugins order: [" + list + "]");
-}
-
-static GstPadProbeReturn event_probe_cb(GstPad* pad,
-                                        GstPadProbeInfo* info,
-                                        gpointer user_data) {
-  if (GST_EVENT_TYPE(GST_PAD_PROBE_INFO_DATA(info)) != GST_EVENT_EOS) {
-    return GST_PAD_PROBE_PASS;
-  }
-
-  gst_pad_remove_probe(pad, GST_PAD_PROBE_INFO_ID(info));
-
-  auto l = static_cast<SinkInputEffects*>(user_data);
-
-  std::lock_guard<std::mutex> lock(l->pipeline_mutex);
-
-  update_order(user_data);
-
-  return GST_PAD_PROBE_DROP;
-}
-
-GstPadProbeReturn on_pad_blocked_eos(GstPad* pad,
-                                     GstPadProbeInfo* info,
-                                     gpointer user_data) {
-  gst_pad_remove_probe(pad, GST_PAD_PROBE_INFO_ID(info));
-
-  auto l = static_cast<SinkInputEffects*>(user_data);
-
-  auto srcpad = gst_element_get_static_pad(
-      l->plugins[l->plugins_order_old[l->plugins_order_old.size() - 1]], "src");
-
-  gst_pad_add_probe(
-      srcpad,
-      static_cast<GstPadProbeType>(GST_PAD_PROBE_TYPE_BLOCK |
-                                   GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM),
-      event_probe_cb, user_data, nullptr);
-
-  auto sinkpad =
-      gst_element_get_static_pad(l->plugins[l->plugins_order_old[0]], "sink");
-
-  gst_pad_send_event(sinkpad, gst_event_new_eos());
-
-  gst_object_unref(sinkpad);
-  gst_object_unref(srcpad);
-
-  return GST_PAD_PROBE_OK;
-}
-
-GstPadProbeReturn on_pad_idle(GstPad* pad,
-                              GstPadProbeInfo* info,
-                              gpointer user_data) {
-  auto l = static_cast<SinkInputEffects*>(user_data);
-
-  std::lock_guard<std::mutex> lock(l->pipeline_mutex);
-
-  update_order(user_data);
-
-  return GST_PAD_PROBE_REMOVE;
-}
-
-void on_plugins_order_changed(GSettings* settings,
-                              gchar* key,
-                              SinkInputEffects* l) {
-  bool update = false;
-  gchar* name;
-  GVariantIter* iter;
-
-  g_settings_get(settings, "plugins", "as", &iter);
-
-  l->plugins_order_old = l->plugins_order;
-  l->plugins_order.clear();
-
-  while (g_variant_iter_next(iter, "s", &name)) {
-    l->plugins_order.push_back(name);
-    g_free(name);
-  }
-
-  g_variant_iter_free(iter);
-
-  if (l->plugins_order.size() != l->plugins_order_old.size()) {
-    update = true;
-  } else if (!std::equal(l->plugins_order.begin(), l->plugins_order.end(),
-                         l->plugins_order_old.begin())) {
-    update = true;
-  }
-
-  if (update) {
-    auto srcpad = gst_element_get_static_pad(l->identity_in, "src");
-
-    GstState state, pending;
-
-    gst_element_get_state(l->effects_bin, &state, &pending,
-                          l->state_check_timeout);
-
-    if (state != GST_STATE_PLAYING) {
-      gst_pad_add_probe(srcpad, GST_PAD_PROBE_TYPE_IDLE, on_pad_idle, l,
-                        nullptr);
-    } else {
-      gst_pad_add_probe(srcpad, GST_PAD_PROBE_TYPE_IDLE, on_pad_blocked_eos, l,
-                        nullptr);
-    }
-
-    g_object_unref(srcpad);
-  }
-}
-
-void on_blocksize_changed(GSettings* settings,
-                          gchar* key,
-                          SinkInputEffects* l) {
-  GstState state, pending;
-
-  gst_element_get_state(l->pipeline, &state, &pending, l->state_check_timeout);
-
-  if (state == GST_STATE_PLAYING || state == GST_STATE_PAUSED) {
-    gst_element_set_state(l->pipeline, GST_STATE_READY);
-
-    l->update_pipeline_state();
+  } else if (src_name == std::string("delay_input_level")) {
+    sie->delay_input_level.emit(sie->get_peak(message));
+  } else if (src_name == std::string("delay_output_level")) {
+    sie->delay_output_level.emit(sie->get_peak(message));
   }
 }
 
 }  // namespace
 
 SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
-    : PipelineBase("sie: ", pulse_manager->apps_sink_info->rate),
-      log_tag("sie: "),
-      pm(pulse_manager),
-      sie_settings(g_settings_new("com.github.wwmm.pulseeffects.sinkinputs")) {
+    : PipelineBase("sie: ", pulse_manager) {
   std::string pulse_props =
       "application.id=com.github.wwmm.pulseeffects.sinkinputs";
 
+  child_settings = g_settings_new("com.github.wwmm.pulseeffects.sinkinputs");
+
   set_pulseaudio_props(pulse_props);
 
   set_source_monitor_name(pm->apps_sink_info->monitor_source_name);
+  set_caps(pm->apps_sink_info->rate);
 
   auto PULSE_SINK = std::getenv("PULSE_SINK");
 
@@ -280,6 +110,8 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
       sigc::mem_fun(*this, &SinkInputEffects::on_app_changed));
   pm->sink_input_removed.connect(
       sigc::mem_fun(*this, &SinkInputEffects::on_app_removed));
+  pm->sink_changed.connect(
+      sigc::mem_fun(*this, &SinkInputEffects::on_sink_changed));
 
   g_settings_bind(settings, "buffer-out", source, "buffer-time",
                   G_SETTINGS_BIND_DEFAULT);
@@ -289,6 +121,7 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
                   G_SETTINGS_BIND_DEFAULT);
   g_settings_bind(settings, "latency-out", sink, "latency-time",
                   G_SETTINGS_BIND_DEFAULT);
+
   g_settings_bind(settings, "blocksize-out", adapter, "blocksize",
                   G_SETTINGS_BIND_DEFAULT);
 
@@ -297,9 +130,6 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
   g_signal_connect(bus, "message::element", G_CALLBACK(on_message_element),
                    this);
 
-  g_signal_connect(settings, "changed::blocksize-out",
-                   G_CALLBACK(on_blocksize_changed), this);
-
   limiter = std::make_unique<Limiter>(
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.limiter");
   compressor = std::make_unique<Compressor>(
@@ -307,7 +137,9 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
   filter = std::make_unique<Filter>(
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.filter");
   equalizer = std::make_unique<Equalizer>(
-      log_tag, "com.github.wwmm.pulseeffects.sinkinputs.equalizer");
+      log_tag, "com.github.wwmm.pulseeffects.sinkinputs.equalizer",
+      "com.github.wwmm.pulseeffects.sinkinputs.equalizer.leftchannel",
+      "com.github.wwmm.pulseeffects.sinkinputs.equalizer.rightchannel");
   reverb = std::make_unique<Reverb>(
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.reverb");
   bass_enhancer = std::make_unique<BassEnhancer>(
@@ -324,6 +156,8 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.loudness");
   gate = std::make_unique<Gate>(log_tag,
                                 "com.github.wwmm.pulseeffects.sinkinputs.gate");
+  pitch = std::make_unique<Pitch>(
+      log_tag, "com.github.wwmm.pulseeffects.sinkinputs.pitch");
   multiband_gate = std::make_unique<MultibandGate>(
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.multibandgate");
   deesser = std::make_unique<Deesser>(
@@ -336,6 +170,8 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.crystalizer");
   autogain = std::make_unique<AutoGain>(
       log_tag, "com.github.wwmm.pulseeffects.sinkinputs.autogain");
+  delay = std::make_unique<Delay>(
+      log_tag, "com.github.wwmm.pulseeffects.sinkinputs.delay");
 
   plugins.insert(std::make_pair(limiter->name, limiter->plugin));
   plugins.insert(std::make_pair(compressor->name, compressor->plugin));
@@ -350,31 +186,32 @@ SinkInputEffects::SinkInputEffects(PulseManager* pulse_manager)
       std::make_pair(multiband_compressor->name, multiband_compressor->plugin));
   plugins.insert(std::make_pair(loudness->name, loudness->plugin));
   plugins.insert(std::make_pair(gate->name, gate->plugin));
+  plugins.insert(std::make_pair(pitch->name, pitch->plugin));
   plugins.insert(std::make_pair(multiband_gate->name, multiband_gate->plugin));
   plugins.insert(std::make_pair(deesser->name, deesser->plugin));
   plugins.insert(std::make_pair(stereo_tools->name, stereo_tools->plugin));
   plugins.insert(std::make_pair(convolver->name, convolver->plugin));
   plugins.insert(std::make_pair(crystalizer->name, crystalizer->plugin));
   plugins.insert(std::make_pair(autogain->name, autogain->plugin));
+  plugins.insert(std::make_pair(delay->name, delay->plugin));
 
   add_plugins_to_pipeline();
 
-  g_signal_connect(sie_settings, "changed::plugins",
-                   G_CALLBACK(on_plugins_order_changed), this);
+  g_signal_connect(child_settings, "changed::plugins",
+                   G_CALLBACK(on_plugins_order_changed<SinkInputEffects>),
+                   this);
 }
 
 SinkInputEffects::~SinkInputEffects() {
-  g_object_unref(sie_settings);
-
   util::debug(log_tag + "destroyed");
 }
 
 void SinkInputEffects::on_app_added(const std::shared_ptr<AppInfo>& app_info) {
   PipelineBase::on_app_added(app_info);
 
-  auto enable_all_apps = g_settings_get_boolean(settings, "enable-all-apps");
+  auto enable_all = g_settings_get_boolean(settings, "enable-all-sinkinputs");
 
-  if (enable_all_apps && !app_info->connected) {
+  if (enable_all && !app_info->connected) {
     pm->move_sink_input_to_pulseeffects(app_info->name, app_info->index);
   }
 }
@@ -384,14 +221,14 @@ void SinkInputEffects::add_plugins_to_pipeline() {
   GVariantIter* iter;
   std::vector<std::string> default_order;
 
-  g_settings_get(sie_settings, "plugins", "as", &iter);
+  g_settings_get(child_settings, "plugins", "as", &iter);
 
   while (g_variant_iter_next(iter, "s", &name)) {
     plugins_order.push_back(name);
     g_free(name);
   }
 
-  auto gvariant = g_settings_get_default_value(sie_settings, "plugins");
+  auto gvariant = g_settings_get_default_value(child_settings, "plugins");
 
   g_variant_get(gvariant, "as", &iter);
 
@@ -409,7 +246,7 @@ void SinkInputEffects::add_plugins_to_pipeline() {
   if (plugins_order.size() != default_order.size()) {
     plugins_order = default_order;
 
-    g_settings_reset(sie_settings, "plugins");
+    g_settings_reset(child_settings, "plugins");
   }
 
   for (auto v : plugins_order) {
@@ -419,7 +256,7 @@ void SinkInputEffects::add_plugins_to_pipeline() {
         default_order.end()) {
       plugins_order = default_order;
 
-      g_settings_reset(sie_settings, "plugins");
+      g_settings_reset(child_settings, "plugins");
 
       break;
     }
