@@ -18,6 +18,8 @@ SinkInputEffectsUi::SinkInputEffectsUi(
       "/com/github/wwmm/pulseeffects/ui/filter.glade");
   auto b_equalizer = Gtk::Builder::create_from_resource(
       "/com/github/wwmm/pulseeffects/ui/equalizer.glade");
+  auto b_pitch = Gtk::Builder::create_from_resource(
+      "/com/github/wwmm/pulseeffects/ui/pitch.glade");
   auto b_reverb = Gtk::Builder::create_from_resource(
       "/com/github/wwmm/pulseeffects/ui/reverb.glade");
   auto b_bass_enhancer = Gtk::Builder::create_from_resource(
@@ -46,6 +48,8 @@ SinkInputEffectsUi::SinkInputEffectsUi(
       "/com/github/wwmm/pulseeffects/ui/crystalizer.glade");
   auto b_autogain = Gtk::Builder::create_from_resource(
       "/com/github/wwmm/pulseeffects/ui/autogain.glade");
+  auto b_delay = Gtk::Builder::create_from_resource(
+      "/com/github/wwmm/pulseeffects/ui/delay.glade");
 
   b_limiter->get_widget_derived(
       "widgets_grid", limiter_ui,
@@ -58,7 +62,11 @@ SinkInputEffectsUi::SinkInputEffectsUi(
       "com.github.wwmm.pulseeffects.sinkinputs.filter");
   b_equalizer->get_widget_derived(
       "widgets_grid", equalizer_ui,
-      "com.github.wwmm.pulseeffects.sinkinputs.equalizer");
+      "com.github.wwmm.pulseeffects.sinkinputs.equalizer",
+      "com.github.wwmm.pulseeffects.sinkinputs.equalizer.leftchannel",
+      "com.github.wwmm.pulseeffects.sinkinputs.equalizer.rightchannel");
+  b_pitch->get_widget_derived("widgets_grid", pitch_ui,
+                              "com.github.wwmm.pulseeffects.sinkinputs.pitch");
   b_reverb->get_widget_derived(
       "widgets_grid", reverb_ui,
       "com.github.wwmm.pulseeffects.sinkinputs.reverb");
@@ -100,11 +108,14 @@ SinkInputEffectsUi::SinkInputEffectsUi(
   b_autogain->get_widget_derived(
       "widgets_grid", autogain_ui,
       "com.github.wwmm.pulseeffects.sinkinputs.autogain");
+  b_delay->get_widget_derived("widgets_grid", delay_ui,
+                              "com.github.wwmm.pulseeffects.sinkinputs.delay");
 
   stack->add(*limiter_ui, limiter_ui->name);
   stack->add(*compressor_ui, compressor_ui->name);
   stack->add(*filter_ui, filter_ui->name);
   stack->add(*equalizer_ui, equalizer_ui->name);
+  stack->add(*pitch_ui, pitch_ui->name);
   stack->add(*reverb_ui, reverb_ui->name);
   stack->add(*bass_enhancer_ui, bass_enhancer_ui->name);
   stack->add(*exciter_ui, exciter_ui->name);
@@ -119,6 +130,7 @@ SinkInputEffectsUi::SinkInputEffectsUi(
   stack->add(*convolver_ui, convolver_ui->name);
   stack->add(*crystalizer_ui, crystalizer_ui->name);
   stack->add(*autogain_ui, autogain_ui->name);
+  stack->add(*delay_ui, delay_ui->name);
 
   // populate_listbox
 
@@ -126,6 +138,7 @@ SinkInputEffectsUi::SinkInputEffectsUi(
   add_to_listbox(compressor_ui);
   add_to_listbox(filter_ui);
   add_to_listbox(equalizer_ui);
+  add_to_listbox(pitch_ui);
   add_to_listbox(reverb_ui);
   add_to_listbox(bass_enhancer_ui);
   add_to_listbox(exciter_ui);
@@ -140,15 +153,40 @@ SinkInputEffectsUi::SinkInputEffectsUi(
   add_to_listbox(convolver_ui);
   add_to_listbox(crystalizer_ui);
   add_to_listbox(autogain_ui);
+  add_to_listbox(delay_ui);
 
   level_meters_connections();
   up_down_connections();
+
+  sie->new_spectrum.connect(
+      sigc::mem_fun(*spectrum_ui, &SpectrumUi::on_new_spectrum));
 }
 
 SinkInputEffectsUi::~SinkInputEffectsUi() {
+  sie->disable_spectrum();
+
   util::debug(log_tag + "destroyed");
 }
 
+SinkInputEffectsUi* SinkInputEffectsUi::add_to_stack(
+    Gtk::Stack* stack,
+    SinkInputEffects* sie_ptr) {
+  auto builder = Gtk::Builder::create_from_resource(
+      "/com/github/wwmm/pulseeffects/ui/effects_base.glade");
+
+  auto settings =
+      Gio::Settings::create("com.github.wwmm.pulseeffects.sinkinputs");
+
+  SinkInputEffectsUi* ui;
+
+  builder->get_widget_derived("widgets_box", ui, settings, sie_ptr);
+
+  stack->add(*ui, "sink_inputs");
+  stack->child_property_icon_name(*ui).set_value("audio-speakers-symbolic");
+
+  return ui;
+}
+
 void SinkInputEffectsUi::level_meters_connections() {
   // limiter level meters connections
 
@@ -161,12 +199,16 @@ void SinkInputEffectsUi::level_meters_connections() {
 
   // compressor level meters connections
 
-  connections.push_back(sie->compressor_input_level.connect(
-      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_input_level_db)));
-  connections.push_back(sie->compressor_output_level.connect(
-      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_output_level_db)));
-  connections.push_back(sie->compressor->compression.connect(
-      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_compression)));
+  connections.push_back(sie->compressor->input_level.connect(
+      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_input_level)));
+  connections.push_back(sie->compressor->output_level.connect(
+      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_output_level)));
+  connections.push_back(sie->compressor->reduction.connect(
+      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_reduction)));
+  connections.push_back(sie->compressor->sidechain.connect(
+      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_sidechain)));
+  connections.push_back(sie->compressor->curve.connect(
+      sigc::mem_fun(*compressor_ui, &CompressorUi::on_new_curve)));
 
   // filter level meters connections
 
@@ -182,6 +224,13 @@ void SinkInputEffectsUi::level_meters_connections() {
   connections.push_back(sie->equalizer_output_level.connect(
       sigc::mem_fun(*equalizer_ui, &EqualizerUi::on_new_output_level_db)));
 
+  // pitch level meters connections
+
+  connections.push_back(sie->pitch_input_level.connect(
+      sigc::mem_fun(*pitch_ui, &PitchUi::on_new_input_level_db)));
+  connections.push_back(sie->pitch_output_level.connect(
+      sigc::mem_fun(*pitch_ui, &PitchUi::on_new_output_level_db)));
+
   // reverb level meters connections
 
   connections.push_back(sie->reverb->input_level.connect(
@@ -330,6 +379,10 @@ void SinkInputEffectsUi::level_meters_connections() {
       sigc::mem_fun(*crystalizer_ui, &CrystalizerUi::on_new_input_level_db)));
   connections.push_back(sie->crystalizer_output_level.connect(
       sigc::mem_fun(*crystalizer_ui, &CrystalizerUi::on_new_output_level_db)));
+  connections.push_back(sie->crystalizer->range_before.connect(
+      sigc::mem_fun(*crystalizer_ui, &CrystalizerUi::on_new_range_before)));
+  connections.push_back(sie->crystalizer->range_after.connect(
+      sigc::mem_fun(*crystalizer_ui, &CrystalizerUi::on_new_range_after)));
 
   // autogain level meters connections
 
@@ -351,6 +404,13 @@ void SinkInputEffectsUi::level_meters_connections() {
       sigc::mem_fun(*autogain_ui, &AutoGainUi::on_new_range)));
   connections.push_back(sie->autogain->gain.connect(
       sigc::mem_fun(*autogain_ui, &AutoGainUi::on_new_gain)));
+
+  // delay level meters connections
+
+  connections.push_back(sie->delay_input_level.connect(
+      sigc::mem_fun(*delay_ui, &DelayUi::on_new_input_level_db)));
+  connections.push_back(sie->delay_output_level.connect(
+      sigc::mem_fun(*delay_ui, &DelayUi::on_new_output_level_db)));
 }
 
 void SinkInputEffectsUi::up_down_connections() {
@@ -401,6 +461,11 @@ void SinkInputEffectsUi::up_down_connections() {
   connections.push_back(filter_ui->plugin_down->signal_clicked().connect(
       [=]() { on_down(filter_ui); }));
 
+  connections.push_back(pitch_ui->plugin_up->signal_clicked().connect(
+      [=]() { on_up(pitch_ui); }));
+  connections.push_back(pitch_ui->plugin_down->signal_clicked().connect(
+      [=]() { on_down(pitch_ui); }));
+
   connections.push_back(equalizer_ui->plugin_up->signal_clicked().connect(
       [=]() { on_up(equalizer_ui); }));
   connections.push_back(equalizer_ui->plugin_down->signal_clicked().connect(
@@ -478,4 +543,9 @@ void SinkInputEffectsUi::up_down_connections() {
       [=]() { on_up(autogain_ui); }));
   connections.push_back(autogain_ui->plugin_down->signal_clicked().connect(
       [=]() { on_down(autogain_ui); }));
+
+  connections.push_back(delay_ui->plugin_up->signal_clicked().connect(
+      [=]() { on_up(delay_ui); }));
+  connections.push_back(delay_ui->plugin_down->signal_clicked().connect(
+      [=]() { on_down(delay_ui); }));
 }
