@@ -105,12 +105,8 @@ StereoToolsUi::StereoToolsUi(BaseObjectType* cobject,
   g_settings_bind_with_mapping(
       settings->gobj(), "mode", mode->gobj(), "active", G_SETTINGS_BIND_DEFAULT,
       stereo_tools_enum_to_int, int_to_stereo_tools_enum, nullptr, nullptr);
-
-  settings->set_boolean("post-messages", true);
 }
 
 StereoToolsUi::~StereoToolsUi() {
-  settings->set_boolean("post-messages", false);
-
   util::debug(name + " ui destroyed");
 }
